# -*- coding: utf-8 -*-
from .sub_rule import SubRule
from .stepped_translator import SteppedTranslator
from .translator import Translator
from .file_translator import FileTranslator
from .translator_cli import TranslatorCLI

__version__ = "0.2.2"
__copyright__ = "2022"
__authors__ = ["Ethosa", "Konard"]
__all__ = [
    'SubRule', 'SteppedTranslator', 'Translator',
    'FileTranslator', 'TranslatorCLI'
]



# -*- coding: utf-8 -*-
from typing import NoReturn, Optional, List
from os import listdir, path, getcwd, mkdir

from .translator import Translator


class FileTranslator(Translator):
    _translator: Optional[Translator] = None
    src_ext: str = ''  # source file extension
    target_ext: str = ''  # target file extension

    def __init__(
        self,
        translator: Translator,
        src_ext: str,
        target_ext: str
    ) -> NoReturn:
        self._translator = translator
        self.src_ext = src_ext
        self.target_ext = target_ext

    def translate(
        self,
        src_path: Optional[str] = None,
        target_path: Optional[str] = None
    ) -> NoReturn:
        """Translates source file
        """
        directory = getcwd()
        if not src_path:
            src_path = directory
        if not target_path:
            target_path = directory
        if path.isdir(src_path) and path.exists(src_path):
            if path.isfile(target_path):
                return
            self.translate_folder(src_path, target_path)
        if path.isfile(src_path) and path.exists(src_path):
            self.translate_file(src_path, target_path)

    def translate_folder(
        self,
        src_path: str,
        target_path: str
    ) -> NoReturn:
        """Translates all folder recursively
        """
        if FileTranslator.files_count(src_path, self.src_ext) == 0:
            return
        if not path.exists(target_path):
            mkdir(target_path)

        directories = [i for i in listdir(src_path) if path.isdir(path.join(src_path, i))]
        files = [i for i in listdir(src_path) if path.isfile(path.join(src_path, i))]

        for i in directories:
            self.translate_folder(path.join(src_path, i), path.join(target_path, i))
        for i in files:
            if i.endswith(self.src_ext):
                self.translate_file(
                    path.join(src_path, i),
                    self.get_target_filename(i, target_path)
                )

    def translate_file(
        self,
        src_file: str,
        target_file: str
    ) -> NoReturn:
        """Translates source file and writes it in target file
        """
        if (path.exists(target_file) and
            path.getmtime(src_file) < path.getmtime(target_file)
        ):
            return
        text = ''
        with open(src_file, 'r', encoding='utf-8') as f:
            text = f.read()
        with open(target_file, 'w', encoding='utf-8') as f:
            f.write(self._translator.translate(text))

    def get_target_filename(
        self,
        src_file: str,
        target_path: str
    ) -> str:
        return path.join(target_path, f'{path.splitext(src_file)[0]}.{self.target_ext}')

    @staticmethod
    def files_count(
        directory: str,
        ext: str
    ) -> int:
        """Returns count of files with specified extension

        :param path: specified path
        :param ext: specified extension
        """
        if not path.exists(directory):
            raise ValueError(f'{directory} not exists.')

        files = [i for i in listdir(directory) if path.isfile(f'{directory}/{i}')]
        directories = [i for i in listdir(directory) if path.isdir(f'{directory}/{i}')]
        result = 0
        for i in directories:
            result += FileTranslator.files_count(f'{directory}/{i}', ext)
        for i in files:
            if i.endswith(ext):
                result += 1
        return result



# -*- coding: utf-8 -*-
from typing import NoReturn, List

from regex import search, sub

from .sub_rule import SubRule


class SteppedTranslator:
    rules: List[SubRule] = []
    text: str = ''
    current: int = 0

    def __init__(
        self,
        rules: List[SubRule] = [],
        text: str = '',
        current: int = 0
    ) -> NoReturn:
        """Initializes a new SteppedTranslator
        """
        self.reset(rules, text, current)

    def reset(
        self,
        rules: List[SubRule] = [],
        text: str = '',
        current: int = 0
    ) -> NoReturn:
        """Changes current properties
        """
        self.rules = rules
        self.text = text
        self.current = current

    def next(
        self
    ) -> bool:
        if len(self.rules) <= self.current:
            return False

        rule = self.rules[self.current]
        replace = -1

        while search(rule.match, self.text) and rule.max_repeat > replace:
            self.text = sub(rule.match, rule.sub, self.text)
            replace += 1

        self.current += 1
        return True



# -*- coding utf-8 -*-
# authors: Ethosa, Konard
from typing import NoReturn, Union, List, Optional
from logging import debug, basicConfig

from regex import Pattern, sub, search

from .sub_rule import SubRule
from .stepped_translator import SteppedTranslator


class Translator:
    rules: List[SubRule] = []

    def __init__(
        self,
        rules: List[SubRule] = []
    ) -> NoReturn:
        """Initializes class.

        :param src: original text.
        :param rules: include your own rules.
        :param debug: enables debug output
        """
        self.rules = rules

    def translate(
        self,
        src: str
    ) -> str:
        """Transforms original text, using specific rules.

        :param src: original text
        :return: transformed text.
        """
        stpd_translator = SteppedTranslator(self.rules, src, 0)
        while stpd_translator.next():
            pass
        return stpd_translator.text



# -*- coding: utf-8 -*-
from typing import NoReturn, Optional

from regex import Pattern, MULTILINE


class SubRule:
    options = MULTILINE  # default regex options
    match: Pattern = r''  # match pattern
    sub: Pattern = r''  # substitution pattern
    path: Optional[Pattern] = None  # path pattern
    max_repeat: int = 0  # maximum repeat count

    def __init__(
        self,
        match: Pattern,
        sub: Pattern,
        path: Optional[Pattern] = None,
        max_repeat: int = 0,
        regex_options: int = options
    ):
        """Initializes Substitution rule.

        :param match: match pattern
        :param sub: substitution pattern
        :param path: path pattern
        :param max_repeat: max match repeat
        :regex_options: regular expression options. by default is Multiline.
        """
        self.match = match
        self.sub = sub
        self.path = path
        self.max_repeat = max_repeat
        self.options = regex_options

    def __str__(
        self
    ) -> str:
        result = f'"{self.match}" -> "{self.sub}"'
        if self.path:
            result = f'{result} on files "{self.path}"'
        if self.max_repeat > 0:
            result = f'{result} repeated {self.max_repeat} times'
        return result



# -*- coding:utf-8 -*-
# authors: Ethosa, Konrad
from typing import NoReturn, Union, List, Optional
import shutil
import os

from regex import Pattern

from .translator import Translator


class TranslatorCLI:
    _translator: Translator
    def __init__(
        self,
        translator: Translator
    ) -> NoReturn:
        """Translator Command Line Interface class.

        :param translator: Translator object
        """
        self._translator = translator

    def run(
        self,
        args: List[str]
    ) -> NoReturn:
        """Runs with the args
        """
        self._translator.translate(
            args[0] if len(args) == 2 else 'input',
            args[1] if len(args) == 2 else 'output'
        )
